@@ -23,21 +23,27 @@ const Gettext = imports.gettext.domain('gnome-shell-extensions');
 const _ = Gettext.gettext;
 const N_ = function(e) { return e };
 
-const POPUP_FADE_TIME = 0.1; // seconds
+const POPUP_DELAY_TIMEOUT = 150; // milliseconds
 
 const SETTINGS_SCHEMA = 'org.gnome.shell.extensions.alternate-tab';
 const SETTINGS_BEHAVIOUR_KEY = 'behaviour';
 const SETTINGS_FIRST_TIME_KEY = 'first-time';
 
 const MODES = {
-    native: function() {
-            Main.wm._startAppSwitcher();
+    all_thumbnails: function(shellwm, binding, mask, window, backwards) {
+        let tabPopup = new AltTabPopup2();
+
+        if (!tabPopup.show(backwards, binding, mask))
+            tabPopup.destroy();
     },
-    all_thumbnails: function() {
-            new AltTabPopup2();
-    },
-    workspace_icons: function() {
-            new AltTabPopupW().show();
+
+    workspace_icons: function(shellwm, binding, mask, window, backwards) {
+        if (shellwm._workspaceSwitcherPopup != null)
+            shellwm._workspaceSwitcherPopup.actor.hide();
+
+        let tabPopup = new AltTabPopupW();
+        if (!tabPopup.show(backwards, binding, mask))
+            tabPopup.destroy();
     }
 };
 
@@ -56,9 +62,8 @@ Workspace & Icons:\n\
     the list and is segregated by a separator/vertical line if available. \n\
     Every window is represented by its application icon.  \n\
 \n\
-Native:\n\
-    This mode is the native GNOME 3 behaviour or in other words: Clicking \n\
-    native switches the Alternate Tab extension off. \n\
+If you whish to revert to the default behavior for the Alt-Tab switcher, just\n\
+disable the extension from extensions.gnome.org or the Advanced Settings application.\
 ");
 
 function AltTabPopupW() {
@@ -68,16 +73,23 @@ function AltTabPopupW() {
 AltTabPopupW.prototype = {
     __proto__ : AltTab.AltTabPopup.prototype,
 
-    show : function(backward, switch_group) {
-        let tracker = Shell.WindowTracker.get_default();
-        let apps = tracker.get_running_apps ('');
+    _windowActivated : function(thumbnailList, n) { },
+
+    show : function(backward, binding, mask) {
+        let appSys = Shell.AppSystem.get_default();
+        let apps = appSys.get_running ();
 
         if (!apps.length)
             return false;
 
-        if (!Main.pushModal(this.actor))
-            return false;
+        if (!Main.pushModal(this.actor)) {
+            // Probably someone else has a pointer grab, try again with keyboard only
+            if (!Main.pushModal(this.actor, global.get_current_time(), Meta.ModalOptions.POINTER_ALREADY_GRABBED)) {
+                return false;
+            }
+        }
         this._haveModal = true;
+        this._modifierMask = AltTab.primaryModifier(mask);
 
         this.actor.connect('key-press-event', Lang.bind(this, this._keyPressEvent));
         this.actor.connect('key-release-event', Lang.bind(this, this._keyReleaseEvent));
@@ -92,16 +104,22 @@ AltTabPopupW.prototype = {
 
         this._appIcons = this._appSwitcher.icons;
 
+        // Need to force an allocation so we can figure out whether we
+        // need to scroll when selecting
+        this.actor.opacity = 0;
+        this.actor.show();
+        this.actor.get_allocation_box();
+
         // Make the initial selection
-        if (switch_group) {
-            if (backward) {
-                this._select(0, this._appIcons[0].cachedWindows.length - 1);
-            } else {
-                if (this._appIcons[0].cachedWindows.length > 1)
-                    this._select(0, 1);
-                else
-                    this._select(0, 0);
-            }
+        if (binding == 'switch_group') {
+            //see AltTab.AltTabPopup.show function
+            //cached windows are always of length one, so select first app and the window
+            //the direction doesn't matter, so ignore backward
+            this._select(0, 0);
+        } else if (binding == 'switch_group_backward') {
+            this._select(0, 0);
+        } else if (binding == 'switch_windows_backward') {
+            this._select(this._appIcons.length - 1);
         } else if (this._appIcons.length == 1) {
             this._select(0);
         } else if (backward) {
@@ -110,24 +128,25 @@ AltTabPopupW.prototype = {
             this._select(1);
         }
 
+
         // There's a race condition; if the user released Alt before
         // we got the grab, then we won't be notified. (See
         // https://bugzilla.gnome.org/show_bug.cgi?id=596695 for
         // details.) So we check now. (Have to do this after updating
         // selection.)
         let [x, y, mods] = global.get_pointer();
-        if (!(mods & Gdk.ModifierType.MOD1_MASK)) {
+        if (!(mods & this._modifierMask)) {
             this._finish();
             return false;
         }
 
-        this.actor.opacity = 0;
-        this.actor.show();
-        Tweener.addTween(this.actor,
-                         { opacity: 255,
-                           time: POPUP_FADE_TIME,
-                           transition: 'easeOutQuad'
-                         });
+        // We delay showing the popup so that fast Alt+Tab users aren't
+        // disturbed by the popup briefly flashing.
+        this._initialDelayTimeoutId = Mainloop.timeout_add(POPUP_DELAY_TIMEOUT,
+                                                           Lang.bind(this, function () {
+                                                               this.actor.opacity = 255;
+                                                               this._initialDelayTimeoutId = 0;
+                                                           }));
 
         return true;
     },
@@ -135,6 +154,9 @@ AltTabPopupW.prototype = {
 
     _finish : function() {
         let app = this._appIcons[this._currentApp];
+        if (!app)
+            return;
+
         Main.activateWindow(app.cachedWindows[0]);
         this.destroy();
     }
@@ -295,13 +317,6 @@ AltTabSettingsDialog.prototype = {
                     this.close();
                 })
             },
-            {
-                label: _("Native"),
-                action: Lang.bind(this, function() {
-                    this.setBehaviour('native');
-                    this.close();
-                })
-            },
             {
                 label: _("Cancel"),
                 action: Lang.bind(this, function() {
@@ -343,23 +358,24 @@ AltTabPopup2.prototype = {
         this._thumbnailTimeoutId = 0;
         this._motionTimeoutId = 0;
 
+
         // Initially disable hover so we ignore the enter-event if
         // the switcher appears underneath the current pointer location
         this._disableHover();
 
-	this.show();
+	//this.show();
         Main.uiGroup.add_actor(this.actor);
-        this._select(0);
+        //this._select(0);
     },
 
-    show : function(backward) {
+    show : function(backward, binding, mask) {
         let windows = global.get_window_actors();
 
 	let list = '';
 	let normal_windows= [];
 	let appIcons = [];
-	let tracker = Shell.WindowTracker.get_default();
-	let apps = tracker.get_running_apps ('');
+	let appSys = Shell.AppSystem.get_default();
+	let apps = appSys.get_running();
 
 	for (let w = windows.length-1; w >= 0; w--) {
 	    let win = windows[w].get_meta_window();
@@ -387,12 +403,19 @@ AltTabPopup2.prototype = {
             }
 	}
 
-        if (!windows.length)
+        if (!windows.length) {
+            this.destroy();
             return false;
+        }
 
-        if (!Main.pushModal(this.actor))
-            return false;
+        if (!Main.pushModal(this.actor)) {
+            // Probably someone else has a pointer grab, try again with keyboard only
+            if (!Main.pushModal(this.actor, global.get_current_time(), Meta.ModalOptions.POINTER_ALREADY_GRABBED)) {
+                return false;
+            }
+        }
         this._haveModal = true;
+        this._modifierMask = AltTab.primaryModifier(mask);
 
         this.actor.connect('key-press-event', Lang.bind(this, this._keyPressEvent));
         this.actor.connect('key-release-event', Lang.bind(this, this._keyReleaseEvent));
@@ -408,10 +431,38 @@ AltTabPopup2.prototype = {
 
         this._appIcons = appIcons;
 
+        // make the initial selection
+        if (backward)
+            this._select(windows.length - 2);
+        else
+            this._select(0);
+
+        this.actor.opacity = 0;
+        this.actor.show();
+
+        // There's a race condition; if the user released Alt before
+        // we got the grab, then we won't be notified. (See
+        // https://bugzilla.gnome.org/show_bug.cgi?id=596695 for
+        // details.) So we check now. (Have to do this after updating
+        // selection.)
+        let [x, y, mods] = global.get_pointer();
+        if (!(mods & this._modifierMask)) {
+            this._finish();
+            return false;
+        }
+
+        // We delay showing the popup so that fast Alt+Tab users aren't
+        // disturbed by the popup briefly flashing.
+        this._initialDelayTimeoutId = Mainloop.timeout_add(AltTab.POPUP_DELAY_TIMEOUT,
+                                                           Lang.bind(this, function () {
+                                                               this.actor.opacity = 255;
+                                                               this._initialDelayTimeoutId = 0;
+                                                           }));
 
 	return true
     },
 
+/*
     _keyPressEvent : function(actor, event) {
         let keysym = event.get_key_symbol();
         let shift = (Shell.get_event_state(event) & Clutter.ModifierType.SHIFT_MASK);
@@ -457,6 +508,7 @@ AltTabPopup2.prototype = {
 
         return true;
     },
+*/
 
     _sortWindows : function(win1,win2) {
         let t1 = win1.get_user_time();
@@ -483,18 +535,18 @@ function WindowList(windows) {
 }
 
 WindowList.prototype = {
-    __proto__ : AltTab.AppSwitcher.prototype,
+    __proto__ : AltTab.SwitcherList.prototype,
 
     _init : function(windows) {
-        AltTab.AppSwitcher.prototype._init.call(this, []);
+        AltTab.SwitcherList.prototype._init.call(this, true);
 
         let activeWorkspace = global.screen.get_active_workspace();
         this._labels = new Array();
         this._thumbnailBins = new Array();
         this._clones = new Array();
         this._windows = windows;
-        this._arrows= new Array();
-        this.icons= new Array();
+        this._arrows = new Array();
+        this.icons = new Array();
 	for (let w = 0; w < windows.length; w++) {
             let arrow = new St.DrawingArea({ style_class: 'switcher-arrow' });
             arrow.connect('repaint', Lang.bind(this, function (area) {
@@ -507,8 +559,8 @@ WindowList.prototype = {
 
 	    let win=windows[w];
 
-	    let tracker = Shell.WindowTracker.get_default();
-	    let apps = tracker.get_running_apps ('');
+	    let appSys = Shell.AppSystem.get_default();
+	    let apps = appSys.get_running();
 	    let ap1 = null;
 	    for (let i = 0; i < apps.length; i++) {
 	        let app_wins = apps[i].get_windows();
@@ -523,15 +575,16 @@ WindowList.prototype = {
                         let clone = new Clutter.Clone ({ source: windowTexture, reactive: true,  width: width * scale, height: height * scale });
                         ap1.icon = ap1.app.create_icon_texture(128);
                         ap1._iconBin.set_size(128,128);
-	                ap1._iconBin.child=clone;
+	                ap1._iconBin.child = clone;
 
-                        ap1.label.text=win.get_title();
+                        ap1.label.text = win.get_title();
 	            }
 	        }
   	    }
             if (ap1 != null) {
-	    ap1.cachedWindows = [win];
-            this._addIcon(ap1);
+	        ap1.cachedWindows = [win];
+                this.addItem(ap1.actor, ap1.label);
+                this.icons.push(ap1);
             }
 	}
     },
@@ -545,29 +598,30 @@ function init(metadata) {
     imports.gettext.bindtextdomain('gnome-shell-extensions', metadata.localedir);
 }
 
-function doAltTab(shellwm, binding, window, backwards) {
+function doAltTab(shellwm, binding, mask, window, backwards) {
     let settings = new Gio.Settings({ schema: SETTINGS_SCHEMA });
 
+
     if(settings.get_boolean(SETTINGS_FIRST_TIME_KEY)) {
         new AltTabSettingsDialog().open();
     } else {
         let behaviour = settings.get_string(SETTINGS_BEHAVIOUR_KEY);
         if(behaviour in MODES) {
-            MODES[behaviour](binding, backwards);
+            MODES[behaviour](shellwm, binding, mask, window, backwards);
         }
     }
 }
 
 function enable() {
-    Main.wm.setKeybindingHandler('switch_windows', doAltTab);
-    Main.wm.setKeybindingHandler('switch_group', doAltTab);
-    Main.wm.setKeybindingHandler('switch_windows_backward', doAltTab);
-    Main.wm.setKeybindingHandler('switch_group_backward', doAltTab);
+    Main.wm.setKeybindingHandler('switch-windows', doAltTab);
+    Main.wm.setKeybindingHandler('switch-group', doAltTab);
+    Main.wm.setKeybindingHandler('switch-windows-backward', doAltTab);
+    Main.wm.setKeybindingHandler('switch-group-backward', doAltTab);
 }
 
 function disable() {
-    Main.wm.setKeybindingHandler('switch_windows', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
-    Main.wm.setKeybindingHandler('switch_group', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
-    Main.wm.setKeybindingHandler('switch_windows_backward', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
-    Main.wm.setKeybindingHandler('switch_group_backward', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
-}
\ No newline at end of file
+    Main.wm.setKeybindingHandler('switch-windows', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
+    Main.wm.setKeybindingHandler('switch-group', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
+    Main.wm.setKeybindingHandler('switch-windows-backward', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
+    Main.wm.setKeybindingHandler('switch-group-backward', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
+}
