@@ -15,11 +15,26 @@ function ThemeManager() {
 
 ThemeManager.prototype = {
     _init: function() {
+    },
+
+    enable: function() {
         this._settings = new Gio.Settings({ schema: SETTINGS_SCHEMA });
-        this._settings.connect('changed::'+SETTINGS_KEY, Lang.bind(this, this._changeTheme));
+        this._changedId = this._settings.connect('changed::'+SETTINGS_KEY, Lang.bind(this, this._changeTheme));
         this._changeTheme();
     },
 
+    disable: function() {
+        if (this._changedId) {
+            this._settings.disconnect(this._changedId);
+            this._changedId = 0;
+        }
+
+        this._settings = null;
+
+        Main.setThemeStylesheet(null);
+        Main.loadTheme();
+    },
+
     _changeTheme: function() {
         let _stylesheet = null;
         let _themeName = this._settings.get_string(SETTINGS_KEY);
@@ -33,7 +48,7 @@ ThemeManager.prototype = {
                 let sysdirs = GLib.get_system_data_dirs();
                 for (let i = 0; i < sysdirs.length; i++) {
                     _userCssStylesheet = sysdirs[i] + '/themes/' + _themeName + '/gnome-shell/gnome-shell.css';
-                    file = Gio.file_new_for_path(_userCssStylesheet);
+                    let file = Gio.file_new_for_path(_userCssStylesheet);
                     if (file.query_exists(null)) {
                         _stylesheet = _userCssStylesheet;
                         break;
@@ -52,6 +67,6 @@ ThemeManager.prototype = {
 }
 
 
-function main(metadata) {
-    new ThemeManager();
+function init(metadata) {
+    return new ThemeManager();
 }
